#include "XojPdfPage.h"

XojPdfRectangle::XojPdfRectangle() = default;

XojPdfRectangle::XojPdfRectangle(double x1, double y1, double x2, double y2): x1(x1), y1(y1), x2(x2), y2(y2) {}

XojPdfPage::XojPdfPage() = default;

XojPdfPage::~XojPdfPage() = default;
