/*
 * Xournal++
 *
 * This is the Windows Crash Handler implementation
 *
 * @author Xournal++ Team
 * https://github.com/xournalpp/xournalpp
 *
 * @license GNU GPLv2 or later
 */

void installCrashHandlers() {
    // Implement SE-Exception Handler for Windows
}
